library(tidyverse)
library(janitor)
library(caTools)
library(caret)
#library(rpart)


setwd("C:/Users/annel/OneDrive - Kruvelab/Kruvelab/Teaching/Data Analysis/Labs/2 Classification")

metabolites <- read_delim("SleepApnia.csv",
                          delim = ",",
                          col_names = TRUE) 

metabolites_metadata <- read_delim("SleepApnia_metadata.csv",
                                   delim = ",",
                                   col_names = TRUE) 

metabolites <- metabolites %>%
  gather(key = "local_sample_id", value = "concentration", 2:37)

metabolites <- metabolites %>%
  spread(key = ` metabolite_name`, value = "concentration")

metabolites <- metabolites %>%
  left_join(metabolites_metadata)

set.seed(123)
#splitting the data
split = sample.split(metabolites$Disease, SplitRatio = 0.8)

train = metabolites %>%
  filter(split == TRUE)

test = metabolites %>%
  filter(split != TRUE)


trControl = trainControl(method = "repeatedcv",
                          number = 5,
                          repeats = 2)

classifier_DT = train(Disease ~ .,
                      data = train %>%
                        select(-local_sample_id, -mb_sample_id), 
                      method = "rpart")

library(rpart.plot)
library(rattle)
#a ready made decision tree visualization
fancyRpartPlot(classifier_DT$finalModel)

train = train %>%
  mutate(Disease_pred_prob = predict(classifier_DT, newdata = train, type = "prob")[2])

library(ROCR)
prediction_quality <- prediction(train$Disease_pred_prob, train$Disease)

summary <- tibble(Cutoff = prediction_quality@cutoffs[[1]], 
                  TruePositives = prediction_quality@tp[[1]],
                  FalsePositives = prediction_quality@fp[[1]],
                  TrueNegatives = prediction_quality@tn[[1]],
                  FalseNegatives = prediction_quality@fn[[1]]) %>%
  mutate(Cutoff = case_when(
    Cutoff == "Inf" ~ 1,
    TRUE ~ Cutoff
  ))

summary <- summary %>%
  mutate(TruePositiveRate = TruePositives / (TruePositives + FalseNegatives),
         FalsePositiveRate = FalsePositives / (FalsePositives + TrueNegatives ))


ggplot(data = summary) +
  geom_line(mapping = aes(x = FalsePositiveRate,
                          y = TruePositiveRate)) +
  geom_abline(slope = 1, intercept = 0)

summary <- summary %>%
  mutate(cost = 3*FalseNegatives + 1*FalsePositives)


train <- train %>%
  mutate(Decision_pred = case_when(
    Disease_pred_prob > 0.41 ~ "sleep apnea",
    TRUE ~ "cardiovascular disease"
  ))

test <- test %>%
  mutate(Disease_pred_prob = predict(classifier_DT, 
                                newdata = test,
                                type = "prob")[2])

test <- test %>%
  mutate(Decision_pred = case_when(
    Disease_pred_prob > 0.41 ~ "sleep apnea",
    TRUE ~ "cardiovascular disease"
  ))

